<?php
// $Id: linktonode.inc,v 1.6 2006/10/22 12:46:47 stborchert Exp $

/**
 * implementation of hook_access()
 **/
function linktonode_access() {
  return array(
    'administer linktonode'
  );
}

/**
 * implementation of hook_menu()
 **/
function linktonode_menu() {
  $mItems = array();
  return $mItems;
}

/**
 * generates a custom settings block for each extension and/or itself
 **/
function linktonode_settings(){
	// set some values on first enter
	if (variable_get('linktonode_extensions', null) === null) {
		variable_set('linktonode_extensions', array(
      'linktonode_default' => array(
        'name' => 'linktonode_default',
        'status' => 1,
        'weight' => -10
      )
      ));
	}
	if (variable_get('linktonode_node_types', null) === null) {
		variable_set('linktonode_node_types', array('story' => 'story', 'page' => 'page'));
	}
  $extension_settings = array();
  $extensions = linktonode_extensions();
  usort($extensions, '_linktonode_extension_compare');

	$form['ext'] = array('#type' => 'fieldset',
		'#collapsible' => true,
		'#collapsed' => true,
		'#tree' => true,
		'#title' => t('Extensions')
  );

	// build extension table
  foreach ($extensions as $key => $extension) {
    $form['ext'][$key]['status'] = array('#type' => 'checkbox', '#default_value' => $extension['status']);
    $form['ext'][$key]['weight'] = array('#type' => 'weight', '#default_value' => $extension['weight']);
    $form['ext'][$key]['title'] = array('#value' => $extension['title']);
    $form['ext'][$key]['name'] = array('#type' => 'hidden', '#value' => $extension['name']);
  }
  $form['opts'] = array('#type' => 'fieldset',
		'#collapsible' => true,
		'#collapsed' => true,
		'#title' => t('Content types'),
		'#description' => t('Select which content types do you want to display in the plugins node listing.'),
		'#tree' => true
  );
	$form['opts']['types'] = array('#type' => 'checkboxes',
		'#description' => t('By selecting a content type all nodes of this type will be displayed in the plugins node listing.
			For example, if you enable \'story\' but not \'page\', only nodes of type \'story\' are listed.'),
		'#options' => array(),
		'#default_value' => variable_get('linktonode_node_types', array())
  );

	$enabled_extensions = variable_get('linktonode_extensions', array());
	usort($enabled_extensions, '_linktonode_extension_compare');
  foreach ($enabled_extensions as $item) {
		if ($item['status'] == 1) {
			$ext = new $item['name']();
			$form['opts']['types']['#options'] += $ext->settings();
			unset($ext);
		}
	}
  return $form;
}

function linktonode_settings_submit($form_id, $values) {
  $extensions = $values['linktonode']['ext'];
  $var = array();
  foreach ($extensions as $ext) {
    $var[$ext['name']] = $ext;
  }
	foreach($var as $key => $ext){
		if ($ext['status'] == 1) {
      if (linktonode_check_dependencies($key)) {
        module_invoke($key, 'settings_submit', $form_id, $values);
      }
      else {
        $var[$key]['status'] = 0;
        $extension = new $key();
        drupal_set_message(t('Extension "%extension" can\'t be enabled for
        						plugin "%plugin" because it depends on modules which are not installed or enabled.<br />
        						Please check <a href="%help">help</a> for more information.',
                    array('%help' => url('admin/help/linktocontent'),
                      '%extension' => $extension->get_title(),
                      '%plugin' => linktonode_get_title())
                    ));
      }
		}
	}
	variable_set('linktonode_extensions', $var);
	variable_set('linktonode_node_types', $values['linktonode']['opts']['types']);
}

function linktonode_get_title() {
  return t('link to node');
}

function linktonode_get_content() {
  $args = func_get_args();
  $args = $args[0];
  $options = array();
  // 1st argument is the selected vocabulary or -1 for list all vocabs
  if (count($args) == 0)
    $vid = -1;
  else
    $vid = $args[0];

  $tid = 0;
  if (isset ($args[1])) {
    $tid = $args[1];
  }
  $options['category'] = _linktonode_get_categories($vid, $tid);
  $options['node'] = array();

	// call linktonode_extensions to ensure the classes are loaded
	linktonode_extensions();
	$enabled_extensions = variable_get('linktonode_extensions', array());
	usort($enabled_extensions, '_linktonode_extension_compare');
  foreach ($enabled_extensions as $item) {
		if ($item['status'] == 1) {
  		$extension = new $item['name']();
			$ext_nodes = $extension->get_nodes($vid, $tid);
			foreach ($ext_nodes as $key => $node) {
        $options['node'][$key] = $node;
			}
			unset($extension);
		}
  }

  return drupal_to_js($options);
}

function linktonode_find_content($search) {
  $options['node'] = array();
  return drupal_to_js($options);
}

function linktonode_extensions() {
  $extension_path = drupal_get_path('module', 'linktocontent') . '/plugins/';
  $extension_path .= 'linktonode/extensions';
  $var = variable_get('linktonode_extensions', array());

  $extensions = file_scan_directory($extension_path, '.*\.inc$', array(
    '.',
    '..',
    'CVS'
  ), 0, false, 'name');
  $result = array();
  foreach ($extensions as $extension_name => $extension) {
    require_once ($extension_path . '/' . $extension->basename);
    $ext = new $extension->name();

    $result[$extension_name]['name'] = $extension_name;
    $result[$extension_name]['title'] = $ext->get_title();
    $result[$extension_name]['status'] = (!isset($var[$extension_name]) ? 0 : $var[$extension_name]['status']);
    $result[$extension_name]['weight'] = (!isset($var[$extension_name]) ? 0 : $var[$extension_name]['weight']);
    unset ($ext);
  }
  return $result;
}

function linktonode_check_dependencies($extension_name) {
  $extension = new $extension_name();
  return $extension->check_dependencies();
}

function linktonode_remove_duplicates($nodes) {
  $result = $nodes;

  return $result;
}

function _linktonode_get_categories($container = -1, $parent = 0) {
  $top = false;
  if ($container <= 0) {
    $container = 0;
    $top = true;
  }

  if (!module_exist('category')) {
    if ($container <= 0){
      $containers = module_invoke('taxonomy', 'get_vocabularies');
    }
    else {
      $containers = module_invoke('taxonomy', 'get_tree', $container, $parent, -1, 1);
    }
  }
  else {
    $cid = $container;
    $obj = null;
    if ($parent > 0)
      $cid = $parent;
    $containers = array();
    $sql = db_rewrite_sql('SELECT c.cid, c.*, h.parent, n.title FROM {category} c INNER JOIN {category_hierarchy} h ON c.cid = h.cid INNER JOIN {node} n ON c.cid = n.nid LEFT JOIN {category} c2 ON h.parent = c2.cid WHERE h.parent = %d AND n.status = 1 AND n.moderate = 0 ORDER BY c.weight, n.title', 'n', 'nid');
    $result = db_query($sql, $cid);
    while ($category = db_fetch_object($result)) {
      $obj = new StdClass;
      $obj->vid = ($category->cnid == 0) ? $category->cid : $category->cnid;
      $obj->tid = $category->cid;
      $obj->name = $category->title;
      $containers[] = $obj;
    }
  }
  if (count($containers) == 0)
    return false;

  $results = array();
  $obj = null;

  foreach ($containers as $item) {
    $obj = new StdClass;
    $obj->vid = $item->vid;
    $obj->tid = ($top ? $item->vid : $item->tid);
    $obj->title = (trim($item->name));
    $results[] = $obj;
  } // end foreach
  return $results;
}

function _linktonode_extension_compare($a, $b) {
  $status = $b['status'] - $a['status'];
  // Separate enabled from disabled.
  if ($status)
    return $status;

  if ($a['status']) {
    return $a['weight'] - $b['weight'];
  }
  else {
    return strcmp($a['title'], $b['title']);
  }
}
?>